import pandas as pd
import requests
import random
from KwoksTool.source.IpPool import GetIpPool
import email
import imaplib
import re
from email.header import decode_header
import execjs
import json
import mimetypes
import smtplib   # 发送邮件模块
from email import encoders
from email.mime.multipart import MIMEMultipart    # 使用MIMEMultipart来标示这个邮件是多个部分组成的
from email.mime.base import MIMEBase
from email.mime.text import MIMEText   # 定义邮件内容
import  datetime
from email.utils import formataddr
import os.path
import sys
from zipfile import ZipFile
import zipfile
import os
def GetCityNumFromLiepin(place):
    # place:城市名称，例如GetCityNumFromLiepin('温州')
    # 返回值:城市代码的数组，例如['300060010', '300060090']
    ip = GetIpPool()
    ip = ip[random.randint(0, len(ip))]
    print('猎聘网正在查询城市编码')
    case=1
    OutNum=1
    while case==1:
        try:
            sheng = place[:place.find('省')]
            shi = place[place.find('省') + 1:place.find('市')]
            url = "https://apidok.liepin.com/api/com.liepin.bd.p.v3.get-all-dq"
            data = "sceneCode=6&from=component"
            header = {
                "Accept": "application/json, text/plain, */*",
                "Content-Type": "application/x-www-form-urlencoded",
                "X-Client-Type": "web",
                "X-Fscp-Fe-Version": "",
                "X-Fscp-Version": "1.1",
                "X-Fscp-Std-Info": "{\"client_id\": \"40108\"}",
                "X-Requested-With": "XMLHttpRequest",
                "X-Fscp-Trace-Id": "8136730b-00b0-4daa-b975-59728e3c2a5a",
                "X-XSRF-TOKEN": "1vL6nJjVRCu01PAwwQmdEg"
            }
            data = requests.post(url, data=data, headers=header).json()
            name = data['data']['data']['city']['list']
            num = data['data']['data']['city']['relations']
            name = str(name)
            citysNum = num[str(((name.split(sheng)[0]).split('\'code\': \'')[-1])[:3])]
            for son in citysNum:
                qu = ((name.split(str(shi))[0]).split('\'code\': \'')[-1])[:6]
                cityName = \
                (((name.split('\'' + str(qu) + '\', \'seoUri\': \'')[1]).split('\', \'category\'')[0])).split('\'')[-1]
                if son == qu:
                    result = num[qu]
                    print('已经为你找到' + sheng + '省' + cityName + '市的所有区县，即将为你查询')
                    break
                else:
                    result = []
            return result
            case=2
        except:
            OutNum=OutNum+1
            if OutNum==10:
                print('抱歉，城市编码查询失败')
                case=2
            else:
                continue
def GetCityNumFromBossZhiPing(city,proxies=False):
    # city:输入省市县的城市名称，例如：'浙江省宁波市'
    # proxies:是否使用Ip代理
    # 返回值：城市代码的数组，例如：['101210400']
    ip = GetIpPool()
    ip = ip[random.randint(0, len(ip))]
    city=(city.split('省')[1]).split('市')[0]
    print('boss直聘查询说明\n本功能只能通过城市名字查询一个，关键词例如(宁波)，返回结果不是区的字典，只是当前城市代码。\n正在查询Boss的城市代码')
    if proxies is True:
        res = requests.get('https://www.zhipin.com/wapi/zpCommon/data/cityGroup.json',proxies=ip).json()
    else:
        res = requests.get('https://www.zhipin.com/wapi/zpCommon/data/cityGroup.json').json()
    res = str(res['zpData']['cityGroup'])
    res = (res.split(str(city))[0]).split('code\': ')[-1].replace(', \'name\': \'', '')
    res=[str(res)]
    return res
def GetCityNameFromLiepin(num,proxies=False):
    # num:城市代码，例如：GetCityNameFromLiepin('300060010',proxies=False)
    # proxies:是否使用代理ip,防止Ip被封禁
    # 返回值：城市名称，例如：阿克苏市
    ip = GetIpPool()
    ip = ip[random.randint(0, len(ip))]
    case = 1
    while case == 1:
        try:
            sheng = num[:num.find('省')]
            shi = num[num.find('省') + 1:num.find('市')]
            url = "https://apidok.liepin.com/api/com.liepin.bd.p.v3.get-all-dq"
            data = "sceneCode=6&from=component"
            header = {
                "Accept": "application/json, text/plain, */*",
                "Content-Type": "application/x-www-form-urlencoded",
                "X-Client-Type": "web",
                "X-Fscp-Fe-Version": "",
                "X-Fscp-Version": "1.1",
                "X-Fscp-Std-Info": "{\"client_id\": \"40108\"}",
                "X-Requested-With": "XMLHttpRequest",
                "X-Fscp-Trace-Id": "8136730b-00b0-4daa-b975-59728e3c2a5a",
                "X-XSRF-TOKEN": "1vL6nJjVRCu01PAwwQmdEg"
            }
            if proxies is True:
                data = requests.post(url, data=data, headers=header, proxies=ip).json()
            else:
                data = requests.post(url, data=data, headers=header).json()
            data = str(data)
            data = data.split('\'sort\': \'' + str(num) + '\'')[1]
            data = data.split('\', \'name\': \'')[1]
            data = data.split('\', \'category\'')[0]
            case = 2
            return data
        except:
            continue
def YesOrNot(ip,case='yes'):
    # 检测代理ip能否用于猎聘网的网页爬取中
    # ip:代理ip
    # 返回值:打印出代理ip和从服务器返回的值，如果返回值里面涵盖工作内容等信息，说明ip可以使用
    def get():
        jsdata = '''
        function r() {
        var e = 1 > 0 && void 0 !== 32 ? 32 : 32;
        console.assert(e >= 32, "最少需生成32位");
        var t = (new Date).getTime()
          , n = e < 32 ? 32 : e
          , r = new Array(n).join("x")
          , a = "".concat(r, "y").replace(/[xy]/g, (function(e) {
            var n = (t + 36 * Math.random()) % 36 | 0;
            return t = Math.floor(t / 36),
            ("x" == e ? n : 3 & n | 32).toString(36)
        }
        ));
        return a
    }'''
        sign = execjs.compile(jsdata).call('r')
        return sign
    def get_data(city, town, work, page, ip,townName):
        ckId_value = get()
        data = '{"data":{"mainSearchPcConditionForm":{"city":"' + str(city) + '","dq":"' + str(town) + '","pubTime":"","currentPage":"' + str(page) + '","pageSize":500,"key":"' + str(
            work) + '","suggestTag":"","workYearCode":"0","compId":"","compName":"","compTag":"","industry":"","salary":"","jobKind":"","compScale":"","compKind":"","compStage":"","eduLevel":"","otherCity":""},"passThroughForm":{"ckId":"' + str(
            ckId_value) + '","scene":"input","skId":"kidyjuir3x7rmen14hlz2kq1y9z6n3i3lqy","fkId":"kidyjuir3x7rmen14hlz2kq1y9z6n3i3lqy","sfrom":"search_job_pc"}}}'
        data = data.encode('utf-8')
        # print(data)
        header = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36",
            "Accept": "application/json, text/plain, */*",
            "Content-Type": "application/json;charset=utf-8;",
            "X-Client-Type": "web",
            "X-Fscp-Fe-Version": "",
            "X-Fscp-Version": "1.1",
            "X-Fscp-Std-Info": "{\"client_id\": \"40108\"}",
            "X-Fscp-Bi-Stat": "{\"location\": \"https://www.liepin.com/zhaopin/?city=410&dq=410&pubTime=&currentPage=0&pageSize=40&key=gs&suggestTag=&workYearCode=0&compId=&compName=&compTag=&industry=&salary=&jobKind=&compScale=&compKind=&compStage=&eduLevel=&otherCity=&sfrom=search_job_pc&scene=input&skId=xne9msayrzxkrk4njg7zq0ym8bfgoueb&fkId=xne9msayrzxkrk4njg7zq0ym8bfgoueb&ckId=" + str(
                ckId_value) + "&suggestId=\"}",
            "X-Requested-With": "XMLHttpRequest",
            "X-Fscp-Trace-Id": "7400ed8f-fa5d-46b3-ae30-c7b1189ded7c",
            "X-XSRF-TOKEN": "q4fNe47HRBup-djOx7S_0Q"
        }
        url = 'https://apic.liepin.com/api/com.liepin.searchfront4c.pc-search-job'
        if case=='yes':
            data = requests.post(url, data=data, headers=header,proxies=ip)
        else:
            data = requests.post(url, data=data, headers=header)
        res = data
        res.encoding = 'utf-8'
        res=res.json()
        print(ip,res)
    get_data('070030','070030010','数据',0,ip,'温州')
def CheckIp(ip):
    # ip:{'http': 'http://39.108.230.16:3128'}
    # 返回值:可以，不可以,返回值为文本
    try:
        if str(ip)[2:7] != 'https':
            res = requests.get('http://httpbin.org/ip', proxies=ip)
            result = '可以'
        else:
            res = requests.get('https://blog.sina.com.cn/', proxies=ip)
            result = '可以'
    except:
        result = '不行'
    return result
def IntoZip(dir_):
    # dir_:需要压缩的文件路径
    with ZipFile(str(dir_) + '.zip', 'w') as z:
        z.write(dir_, arcname=(dn := os.path.basename(dir_)))
        for root, dirs, files in os.walk(dir_):
            for fn in files:
                z.write(
                    fp := os.path.join(root, fn),
                    arcname=dn + '/' + os.path.relpath(fp, dir_)
                )
    print('压缩成功，已经压缩到了文件夹:' + str(dir_) + '.zip')
def ZipOut(dir, out):
    # dir:解压文件的路径，涵盖文件名
    # out:解压到
    zf = zipfile.ZipFile(dir)
    os.chdir(out)
    ret = zf.extractall(path=out)  # 解压到指定目录
    print('解压成功')
def SendEmail(name,password, receivers, text, sub, filepath):
    smtp_server = "smtp.qq.com"  # 发送邮箱服务器
    username = "yongxingkwok@foxmail.com"  # 用于发送邮箱的用户账号
    sender = 'yongxingkwok@foxmail.com'  # 发送者的邮箱
    EMAIL_FROM_NAME = name  # 自定义发件人名称
    time = datetime.datetime.today().strftime("%m-%d %H：%M")
    msg = MIMEMultipart()
    # 邮件正文
    msg.attach(MIMEText(str(text), 'plain', 'utf-8'))  # 文本内容换行\r\n
    msg['From'] = formataddr(pair=(EMAIL_FROM_NAME, sender))  # 自定义发件人的名称
    msg['To'] = ";".join(receivers)  # 发送给多个好友
    subject = str(sub).format(time)
    msg['Subject'] = subject
    if filepath != '':
        data = open(filepath, 'rb')
        ctype, encoding = mimetypes.guess_type(filepath)
        if ctype is None or encoding is not None:
            ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        file_msg = MIMEBase(maintype, subtype)
        file_msg.set_payload(data.read())
        data.close()
        encoders.encode_base64(file_msg)  # 把附件编码
        file_msg.add_header('Content-Disposition', 'attachment',
                            filename=str(filepath.split('\\')[-1].split('.')[0]))  # 修改邮件头
        msg.attach(file_msg)
    server = smtplib.SMTP(smtp_server)
    server.login(username, password)
    server.sendmail(sender, receivers, msg.as_string())
    server.quit()
def GetEmail(mail, password, dir):
    def decode_str(s):  # 字符编码转换
        value, charset = decode_header(s)[0]
        if charset:
            value = value.decode(charset)
            return value

    def get_email_title(msg):
        subject = email.header.decode_header(msg.get('subject'))
        if type(subject[-1][0]) == bytes:
            title = subject[-1][0].decode(str(subject[-1][1]))
        elif type(subject[-1][0]) == str:
            title = subject[-1][0]
        print("title:", title)
        return title

    def get_att(msg, dir):
        for part in msg.walk():
            # file_name = part.get_param("name")
            file_name = part.get_filename()
            if file_name:
                h = email.header.Header(file_name)
                dh = email.header.decode_header(h)
                filename = dh[0][0]
                if dh[0][1]:
                    filename = decode_str(str(filename, dh[0][1]))
                    data = part.get_payload(decode=True)
                    att_file = open(dir + file_name + '.zip', 'wb')
                    print('附件下载成功',file_name)
                    att_file.write(data)
                    att_file.close()

    """入口函数，登录imap服务"""
    server = imaplib.IMAP4_SSL('smtp.qq.com', 993)
    server.login(mail, password)
    server.select('INBOX')
    status, data = server.search(None, "ALL")
    if status != 'OK':
        raise Exception('read email error')
    emailids = data[0].split()
    mail_counts = len(emailids)
    # print("count:", mail_counts)
    # 邮件的遍历是按时间从后往前，这里我们选择最新的一封邮件
    for i in range(mail_counts - 1, mail_counts - 2, -2):
        status, edata = server.fetch(emailids[i], '(RFC822)')
        msg = email.message_from_bytes(edata[0][1])
        # 获取邮件主题title
        subject = email.header.decode_header(msg.get('subject'))
        if type(subject[-1][0]) == bytes:
            title = subject[-1][0].decode(str(subject[-1][1]))
        elif type(subject[-1][0]) == str:
            title = subject[-1][0]
        # print("title:", title)
        get_att(msg, dir)
def ProgressBar(i, sum, printvalue):
    sys.stdout.write('\r')
    sys.stdout.write('{}%|{}{}|{}'.format(int(i / sum * 100), ((int(i / sum * 100))) * '■',
                                          (100 - int(i / sum * 100 + 1)) * '_', '当前打印：' + str(printvalue)))
    sys.stdout.flush()
def MergeTable(data1, data2, on, data1name, data2name):
    data1 = pd.merge(data1, data2, on=on, how='outer')
    for i in range(0, len(list(data1))):
        data1 = data1.rename(
            columns={list(data1)[i]: list(data1)[i].replace('_x', data1name).replace('_y', data2name)})
    return data1
def ChoiceColumn(date,name):
    date1=date.copy()
    if len(name)==0:
        print(list(date))
    else:
        for i in range(0,len(list(date))):
            if list(date)[i] in name:
                pass
            else:
                del(date1[list(date)[i]])
    return date1
def PackageWithoutBorderBorder(file_path=r'python.py',ico_path='source\python.ico'):
    if os.path.exists(os.path.join(os.getcwd(),r'dist\python.exe')) is True:
        os.remove(os.path.join(os.getcwd(),r'dist\python.exe'))
    os.system('pyinstaller -F -w -i ' + os.path.join(os.getcwd(), ico_path) + ' ' + os.path.join(os.getcwd(), file_path))
def Package(file_path=r'python.py',ico_path='source\python.ico'):
    if os.path.exists(os.path.join(os.getcwd(),r'dist\python.exe')) is True:
        os.remove(os.path.join(os.getcwd(),r'dist\python.exe'))
    os.system('pyinstaller -F -i '+os.path.join(os.getcwd(),ico_path)+' '+os.path.join(os.getcwd(),file_path))
def DeleteAll():
    os.system('pip freeze>modules.txt')
    print('pip uninstall -r modules.txt -y')
def PackageService():
    if os.path.exists(os.path.join(os.getcwd(),r'dist\python.exe')) is True:
        os.remove(os.path.join(os.getcwd(),r'dist\python.exe'))
    code = 'pyinstaller -F -w --add-data="' + str(
        os.getcwd()) + '\\venv\Lib\site-packages\pyecharts;pyecharts"  --add-data="' + str(
        os.getcwd()) + '\\venv\Lib\site-packages\\flask;flask" --add-data=templates;templates --add-data=static;static -i ' + str(
        os.getcwd()) + '\\source\python.ico ' + str(os.getcwd()) + '\服务器后端.py'
    os.system(code)
def array_translation(date,num):
    # 数组平移函数，date表示需要平移的数组，num表示平移的个数
    date1=date[0:num]
    date2=date[num:len(date)]
    date=date2+date1
    return date
def work_path():
    name = str(os.getcwd())
    name = re.findall('.*\d*-[\u4e00-\u9fa5]*', name)
    os.chdir(name[0])
    import sys
    sys.path.append(os.getcwd())


import json
import time
import shutil
from KwoksTool.spider import Browser
from KwoksTool.function import (ProgressBar,ChoiceColumn,GetIpPool)
import pandas as pd
import requests
import numpy as np
from lxml import etree
import os
import re
import random
def GetStocksData(year,array,path,ip=False,type='all'):
    # 函数说明：本函数主要用于采集同花顺行业板块的各类股票数据，可实现一次性采集所有行业；采集单个股票；
    # 参数说明
    # （一）year:采集数据的目标年份的数组，例如：year = [2023, 2022, 2021, 2020, 2019]
    # （二）array:单个采集的代码数组，例如：array = ['1A0001','399001']
    # （三）path:采集完的数据保存的位置，例如：path=r'.\板块数据'
    # （四）ip,是否使用代理Ip，True表示使用
    # （五）type:表示函数功能类型，其中
    # （1）type='all'时：采集所有行业板块的数据
    # （2）type='singel'时：仅仅采集array数组里面的数据
    # （3）type='sum'时：对采集的数据进行合并
    # （六）注意
    # （1）采集过程中ip大概率会被封，无法采集以后退出程序重新运行，如果还是不行就使用代理ip
    # (2)采集，合并，单独采集，需要分开一次一次的运行，采集完以后，最后的合并只需一次
    def spider(num,year,ip=None,type='bk'):
        head={
            'User-Agent':'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36',
            'Cookie':'log=; Hm_lvt_722143063e4892925903024537075d0d=1687780092,1687863312; Hm_lvt_929f8b362150b1f77b477230541dbbc2=1687780092,1687863312; Hm_lvt_78c58f01938e4d85eaf619eae71b4ed1=1687780092,1687863312; v=AwiFzmPt-LFA3RQ1tmT6rwUk2X0fsW6UThZALcK5V192z6ajasE8S54lEK0R'
        }
        url='http://d.10jqka.com.cn/v4/line/'+str(type)+'_'+str(num)+'/01/'+str(year)+'.js'
        print(url)
        if ip is not None:
            df=requests.get(url,headers=head,proxies=ip)
        else:
            df = requests.get(url, headers=head)
        date=df.text
        date=date.split('{"data":"')[1].split('"})</pre>')[0]
        date=date.split(';')
        result=[]
        for i in range(len(date)):
            son=date[i].split(',')
            result_son=[]
            for son in son:
                result_son.append(son)
            result.append(result_son)
        return result
    def choose():
        # 调用此函数可以获取A股股票的行业分类代码汇总
        # 其中，生成本地文件到资料\板块代码中
        path=r'.\临时资料\板块代码.txt'
        if os.path.exists(path):
            with open(path,encoding='utf-8') as f:
                f=f.read()
                f=f.replace(' ','').replace('\'','"')
                date=json.loads(f)
            return date
        else:
            def plate():
                date = Browser('https://q.10jqka.com.cn/thshy/index/field/199112/order/desc/page/1/ajax/1/',show=True)
                date=date.page_source
                date=etree.HTML(date)
                dates=date.xpath('/html/body/table/tbody/tr')
                record=[]
                for i in range(len(dates)):
                    date_num=(dates[i].xpath('./td[2]/a/@href'))[0]
                    date_num=date_num.split('/')[-2]
                    date_name=dates[i].xpath('./td[2]/a')[0]
                    date_name=date_name.text
                    record.append([str(i+1),date_name,date_num])
                # ======================
                date = Browser('https://q.10jqka.com.cn/thshy/index/field/199112/order/desc/page/2/ajax/1/',show=True)
                date=date.page_source
                date=etree.HTML(date)
                dates=date.xpath('/html/body/table/tbody/tr')
                for ii in range(len(dates)):
                    date_num=(dates[ii].xpath('./td[2]/a/@href'))[0]
                    date_num=date_num.split('/')[-2]
                    date_name=dates[ii].xpath('./td[2]/a')[0]
                    date_name=date_name.text
                    record.append([str(ii+i+1),date_name,date_num])
                # ===================================
                return record
            date=plate()
            with open(path,'w',encoding='utf-8') as f:
                f.write(str(date))
            return date
    def missing(years):
        path=r'.\临时资料\板块\采集'
        names=choose()
        dir=[]
        for name in names:
            for year in years:
                dir.append(name[2]+'-'+str(year)+'.xlsx')
        dirs=(np.sort(os.listdir(path))).tolist()
        need_again=[]
        for i in range(len(dir)):
            if dir[i] not in dirs:
                need_again.append(dir[i])
        return need_again
    def pick(array,year,time_s=1,ip=None,type='bk'):
        if len(array[0])==3:
            def pick_single(year, num):
                for i in year:
                    try:
                        a = spider(num, i,ip=ip,type=type)
                        a = pd.DataFrame(a,columns=['时间','开盘','最高','最低','收盘','总手','金额','','','',''])
                        a.to_excel(r'.\临时资料\板块\采集\\' + str(num) + '-' + str(i) + '.xlsx')
                        time.sleep(time_s)
                    except:
                        # print(i, num)
                        print('\n错误：https://d.10jqka.com.cn/v4/line/bk_' + str(num) + '/01/' + str(i) + '.js')
                        continue
                        time.sleep(time_s)
            for i in range(len(array)):
                array[i]=array[i][2]
                pick_single(year,array[i])
                print(i,len(array)-1,array[i],year[i])
        else:
            def pick_again(num, da):
                (pd.DataFrame(spider(num, da,ip=ip,type=type),columns=['时间','开盘','最高','最低','收盘','总手','金额','','','',''])).to_excel(r'.\临时资料\板块\采集\\' + str(num) + '-' + str(da) + '.xlsx')
                time.sleep(time_s)
            # print('下面为第二次采集的数据')
            for i in range(len(array)):
                print(i,len(array)-1)
                try:
                    pick_again(array[i][0:6],array[i][7:11])
                except:
                    continue
                    time.sleep(time_s)
    def check(year):
        def to_str(year):
            yea=[]
            for son in year:
                yea.append(str(son))
            return yea
        row=[]
        column=year
        date_check=[]
        path=r'.\临时资料\板块\采集'
        files=os.listdir(path)
        names=choose()
        for name in names:
            row.append(name[2])
        for row_son in row:
            date_column_son = []
            for column_son in column:
                if str(row_son)+'-'+str(column_son)+'.xlsx' in files:
                    date_column_son.append(1)
                else:
                    date_column_son.append(0)
            date_check.append(date_column_son)
        (pd.DataFrame(date_check,index=row,columns=to_str(year))).to_excel(r'.\临时资料\板块\采集汇总.xlsx')
        print('采集汇总文件已经保存到\临时资料\板块\采集汇总.xlsx中')
    def sum(path=None,num=None):
        def Merge(num):
            def find_need(date,case):
                result=[]
                for son in date:
                    if case in son:
                        result.append(son)
                return result
            source_path=r'.\临时资料\板块\采集'
            result_path=r'.\临时资料\板块\合并'
            names=find_need(np.sort(os.listdir(source_path)),num)
            file_date=pd.DataFrame()
            for name in names:
                file_path=os.path.join(source_path,name)
                file_date=pd.concat([file_date,ChoiceColumn((pd.read_excel(file_path)),['时间','开盘','最高','最低','收盘','总手','金额'])])
            file_date=file_date.reset_index(drop=True)
            file_name=r'.\\'+num+'.xlsx'
            file_date.to_excel(os.path.join(result_path,file_name))
        if num is None:
            industrys=choose()
            for i in range(len(industrys)):
                Merge(industrys[i][2])
                ProgressBar(i,len(industrys)-1,'正在合并数据'+str(industrys[i][2]))
        else:
            for i in range(len(num)):
                Merge(num[i])
        if path is not None:
            shutil.move(r'.\临时资料\板块\合并',path)
    def DataCollectionPlate(year,ip=False):
        # 输入年份的参数后采集A股板块的所有股票的数据
        # year = [2023, 2022, 2021, 2020, 2019]
        # ip:参数为false时使用代理ip,True时候不使用代理
        # ===================================================
        if len(os.listdir(r'.\临时资料\板块\采集')) == 0:
            print('首次采集，如果首次无法采集任意一个请使用代理Ip')
            array = choose()
            pick(array, year)
            check(year)
        else:
            print('补充采集')
            array = missing(year)
            if ip is not False:
                ip = (GetIpPool())[random.randint(0, 1500)]
                print(ip)
            pick(array, year, ip=ip)
            check(year)
        print('当前完成率为：' + str(
            int((len(os.listdir(r'.\临时资料\板块\采集')) / (len(choose()) * len(year))) * 10000) / 100) + '%')
    def SpecificStocksData(array,year,time_sleep=1):
        # 下载单独的特定代码的股票数据
        # year = [2023, 2022, 2021, 2020, 2019]
        # array=['1A0001','1A0002']
        # ==================================
        for array_son in array:
            for year_son in year:
                print(str(array_son)+'-'+str(year_son))
                date=spider(str(array_son),str(year_son),type='zs')
                (pd.DataFrame(date,columns=['时间','开盘','最高','最低','收盘','总手','金额','','','',''])).to_excel(r'.\临时资料\板块\采集\\' + str(array_son) + '-' + str(year_son) + '.xlsx')
                time.sleep(time_sleep)
        # 合并数据
        def Merge(num):
            def find_need(date,case):
                result=[]
                for son in date:
                    if case in son:
                        result.append(son)
                return result
            source_path=r'.\临时资料\板块\采集'
            result_path=r'.\临时资料\板块\合并'
            names=find_need(np.sort(os.listdir(source_path)),num)
            file_date=pd.DataFrame()
            for name in names:
                file_path=os.path.join(source_path,name)
                file_date=pd.concat([file_date,ChoiceColumn((pd.read_excel(file_path)),[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10])])
            file_date=file_date.reset_index(drop=True)
            file_name=r'.\\'+num+'.xlsx'
            file_date.to_excel(os.path.join(result_path,file_name))
        for i in range(len(array)):
            Merge(array[i])
    def main(year,array,type,path,ip=ip):
        print('懒得修复Bug的警告提示：\n当type类型为single时候，如果只是采集这个，会出问题。所以，函数类型的调用必须按照如下顺序：\n\
        all->single->sum\n')
        if type=='all':
            DataCollectionPlate(year, ip=ip)
        elif type=='single':
            SpecificStocksData(array, year)
        elif type=='sum':
            sum()
            sum(path=path,num=array)
            shutil.rmtree(r'.\临时资料')
    def File():
        path1=r'.\临时资料\板块\采集'
        path2 = r'.\临时资料\板块\合并'
        if  not os.path.exists(path1):
            os.makedirs(path1)
        if not os.path.exists(path2):
            os.makedirs(path2)
    File()
    main(year,array,type,path)


import pandas as pd
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from datetime import datetime
import tushare as ts
import re
from KwoksTool.function import (ProgressBar,ChoiceColumn)
def Browser(url, show=False):
    options = webdriver.ChromeOptions()
    options.add_experimental_option("excludeSwitches", ["enable-automation"])
    options.add_experimental_option('useAutomationExtension', False)
    if show is False:
        options.add_argument('--headless')
    driver = webdriver.Chrome(options=options, service=Service("chromedriver.exe"))
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": """
      Object.defineProperty(navigator, 'webdriver', {
        get: () => undefined
      })
    """
    })
    # driver.maximize_window()
    driver.minimize_window()
    driver.get(url)
    return driver
def PlateComponentStocks(PlateInfoName, url,code):
    # code:tushure的秘钥
    # url:文件保存的路径
    # plateinfoname:股票的板块代码
    def GetPlateSon(stock_num):
        def get_stock_name(date, num):
            def get_re_name(date):
                date = '<td><a href="http://stockpage.10jqka.com.cn/' + date + '" target="_blank">\D*</a'
                return date

            a = (re.findall(re.compile(str(get_re_name(str(num)))), date))
            '''<td><a href="http://stockpage.10jqka.com.cn/000001" target="_blank">平安银行</a>'''
            a = re.findall(r'>\D*</a', a[0])[0]
            a = a.replace('>', '').replace('</a', '')
            return a

        date = (Browser('http://q.10jqka.com.cn/thshy/detail/field/199112/order/desc/page/1/ajax/1/code/' + stock_num,
                        show=True)).page_source
        date_arr = re.findall(r'\d{6}<', date)
        web_pages = date
        date_name = []
        if len((re.findall(r'>\d*/\d*<', date))) == 0:
            for i in range(len(date_arr)):
                date_arr[i] = date_arr[i].replace('<', '')
                date_name.append(get_stock_name(date, date_arr[i]))
            return date_arr
        else:
            page = (re.findall(r'>\d*/\d*<', date))[0].split('/')[1].replace('<', '')
            for i in range(2, int(page) + 1):
                web_page = (Browser(
                    'http://q.10jqka.com.cn/thshy/detail/field/199112/order/desc/page/' + str(i) + '/ajax/' + str(
                        i) + '/code/' + stock_num, show=True)).page_source
                web_pages = web_pages + web_page
                date_arr = date_arr + re.findall(r'\d{6}<', web_page)
            for i in range(len(date_arr)):
                date_arr[i] = date_arr[i].replace('<', '')
                date_name.append(get_stock_name(web_pages, date_arr[i]))
            result = {'代码': date_arr, '名称': date_name}
            return result
    def FindCode(codes):
        result = []
        pro = ts.pro_api(ApiCode)
        data = pro.daily(ts_code='')
        data = data.sort_values(by='ts_code')
        data = data['ts_code'].T.values.tolist()
        for i in range(0, len(data)):
            if data[i][:6] == codes:
                result.append(data[i])
        return result
    a =GetPlateSon(PlateInfoName)
    ApiCode=code
    pro = ts.pro_api(ApiCode)
    print('正在下载' + str(PlateInfoName) + '板块的股票信息')
    for i in range(0, len(a['代码'])):
        try:
            code = FindCode(a['代码'][i])
            for son in code:
                data = pro.daily(ts_code=son)
                data = data.rename(
                    columns={'open': '开盘', 'close': '收盘', 'high': '最高', 'low': '最低', 'amount': '总手',
                             'vol': '金额', 'trade_date': '时间'})
                data = ChoiceColumn(data, ['开盘', '最高', '最低', '收盘', '总手', '金额', '时间'])
                data = data.sort_values(by='时间')
                data = pd.DataFrame(data, columns=['开盘', '最高', '最低', '收盘', '总手', '金额', '时间'])
                data = data.reset_index(drop=True)
                # ==========================================
                data.to_excel(r'{}\{}.xlsx'.format(url, a['代码'][i]))
                ProgressBar(i, len(a['代码']), a['名称'][i])
        except:
            print(i,'错误')
            continue


import numpy as np
def GetProbMatrix(df):
    name = []
    for son in df:
        if son not in name:
            name.append(son)
    result = []
    name = np.sort(name)
    for i in range(0, len(name)):  # 遍历行
        result.append([])
        for ii in range(0, len(name)):  # 遍历列
            result[i].append(0)
            # input(result)
            for iii in range(1, len(df)):
                if df[iii - 1] == name[i] and df[iii] == name[ii]:
                    result[i][ii] = result[i][ii] + 1
        sum = np.sum(result[i])
        for iiii in range(len(result[i])):
            result[i][iiii] = result[i][iiii] / sum
        # print('sum is ',np.sum(result[i]),'序列',result[i],'\n')
    result = np.array(result)
    return result
def ToMat(df):
    wide = df.shape[1]
    high = df.shape[0]
    df = df.values
    result = []
    size = []
    for i in range(0, high):
        for ii in range(0, wide):
            result.append([[df[i][ii]]])
        size.append(wide)
    back = [np.concatenate(result), size]
    return back